function [totalVol] = rectangleCylinderIntersectVolumes(pitR, pitDepth, grainL, grainXS, pitOffset, verbose)

if nargin<6
    verbose = 0;
end

if verbose
    figure; hold on
    viscircles([pitOffset, 0], pitR);
    rectangle('Position', [-grainL/2, -grainXS/2, grainL, grainXS])
    axis equal
end

realDepth = min(pitDepth, grainXS/2);

% start with circular pit XSection, then we'll remove the parts that extend
% beyond the grain
XSarea = pi*pitR^2;

%% first check for no overlap/0 radius

if grainL/2<=pitOffset-pitR || -grainL/2>=pitOffset+pitR || pitR==0
    totalVol = 0;
    if verbose
        disp('No overlap, 0 volume')
    end
    return
end

%% next check simple case - grain>pit
if grainXS/2>pitR && grainL/2>pitOffset+pitR && -grainL/2<pitOffset-pitR
    totalVol = XSarea*realDepth;
    if verbose
        disp('Grain Larger than pit, area is pit circle')
    end
    return
end

%% last simple case, grain is entirely within pit
% actually need to check a bunch here - need pitR to be less than width/2
% and the edges of the pit to be less than the grain length and all four
% grain corners to be in the pit
if pitR>grainXS/2 && checkPointInCircle([grainL/2, grainL/2, -grainL/2, -grainL/2], [grainXS/2, 0, grainXS/2, 0], pitR, pitOffset)
    totalVol = grainXS*grainL*realDepth;
    if verbose
        disp('Grain fits inside pit, area is grain rectangle')
    end
    return
end

%% next cases: pit overlaps grain end face (but not to corner) & does not overlap sides
if checkPointInCircle(grainL/2, 0, pitR, pitOffset) && (pitR<=grainXS/2 || pitOffset>grainL/2) && ...
        ~checkPointInCircle(grainL/2, grainXS/2, pitR, pitOffset)
    segmentArea = calculateSegmentArea(pitR,grainL/2-(pitOffset-pitR));
    totalVol = segmentArea*realDepth;
    if verbose
        disp('Overlap only at pos. end, area is overlapping segment')
    end
    return
elseif checkPointInCircle(-grainL/2, 0, pitR, pitOffset, 1) && pitR<=grainXS/2 && ...
        ~checkPointInCircle(-grainL/2, grainXS/2, pitR, pitOffset, 1)
    segmentArea = calculateSegmentArea(pitR, (pitOffset+pitR)-(-grainL/2));
    totalVol = segmentArea*realDepth;
    if verbose
        disp('Overlap only at neg. end, area is overlapping segment')
    end
    return
end
    

%% next deal with the pit extending over grain sides, this is easier because of symmetry
if pitR*2>grainXS % I think this should always happen now?
    sideSegment = calculateSegmentArea(pitR, pitR-grainXS/2);
    XSarea = XSarea-sideSegment*2;
    if verbose
        disp('Removing side segments')
    end
end    

%% check if grain does not extend past ends (so only goes past sides)
if ~checkPointInCircle([-grainL/2 grainL/2], [0 0], pitR, pitOffset, 0)
    totalVol = XSarea*realDepth;
    if verbose
        disp('Pit extends past sides but not edges')
    end
    return
end

%% grain also extends past either end, but the segments don't overlap
% can identify this case now when the corners are not in the circle (other
% times this is possible should be already caught)
if ~checkPointInCircle([-grainL/2 grainL/2], [grainXS/2, grainXS/2], pitR, pitOffset, 0)
    if ~checkPointInCircle(-grainL/2, grainXS/2, pitR, pitOffset) && ...
            checkPointInCircle(-grainL/2, 0, pitR, pitOffset)
        botSegment = calculateSegmentArea(pitR, -grainL/2-(pitOffset-pitR));
        XSarea = XSarea-botSegment;
        if verbose
            disp('Removing bottom segment')
        end
    end
    if ~checkPointInCircle(grainL/2, grainXS/2, pitR, pitOffset) && ...
            checkPointInCircle(grainL/2, 0, pitR, pitOffset)
        topSegment = calculateSegmentArea(pitR, (pitOffset+pitR)-grainL/2);
        XSarea = XSarea-topSegment;
        if verbose
            disp('Removing top segment')
        end
    end
    totalVol = XSarea*realDepth;
    if verbose
        disp('Pit extends past top or bottom, does not overlap side segments')
    end
    return
end

%% last case: grain extends past one end and sides, with overlapping segments
if checkPointInCircle([grainL/2 grainL/2], [grainXS/2, 0], pitR, pitOffset)
    rectEnd = sqrt(pitR^2-(grainXS/2)^2)+pitOffset;
    rectArea = grainXS*(rectEnd-grainL/2);
    edgeSegment = calculateSegmentArea(pitR, pitOffset+pitR-rectEnd);
    if verbose
        disp('Top not in grain, overlaps with side segments')
        rectangle('Position', [grainL/2, -grainXS/2, rectEnd-grainL/2, grainXS], 'FaceColor', 'b')
    end
    XSarea = XSarea-rectArea-edgeSegment;
    totalVol = XSarea*realDepth;
    return
end
if checkPointInCircle([-grainL/2 -grainL/2], [grainXS/2, 0], pitR, pitOffset)
    rectEnd = -sqrt(pitR^2-(grainXS/2)^2)+pitOffset;
    rectArea = grainXS*(-grainL/2-rectEnd);
    edgeSegment = calculateSegmentArea(pitR, rectEnd-(pitOffset-pitR));
    if verbose
        disp('Bottom not in grain, overlaps with side segments')
        rectangle('Position', [rectEnd, -grainXS/2, -grainL/2-rectEnd, grainXS], 'FaceColor', 'b')
    end
    XSarea = XSarea-rectArea-edgeSegment;
    totalVol = XSarea*realDepth;
    return
end

error('We should not have gotten here!')


end

function [segmentVol, secantLength] = calculateSegmentArea(r, h)

% calculate the segment ('chord') volume with height h above the base of 
% a circle with radius r
%
% also calculates the secant length, which we need to know if the sides and
% top intersect

if h<0
    error('Negative segment H!!')
end

if h>r
%    warning('h>radius!')
    h = 2*r-h;
    flipSegVol = true;
else
    flipSegVol = false;
end

sectorVol = acos((r-h)/r)*r^2;

insetTriVol = (r-h)*sqrt(2*r*h-h^2);

segmentVol = sectorVol-insetTriVol;

if flipSegVol
    segmentVol = pi*r^2-segmentVol;
end

secantLength = 2*sqrt(2*r*h-h^2);

end

function [inCircle] = checkPointInCircle(X,Y,r,Xoffset, allIn)

if nargin<5
    allIn = true;
end

if allIn
    inCircle = all((X-Xoffset).^2+Y.^2<r^2);
else
    inCircle = any((X-Xoffset).^2+Y.^2<r^2);
end

end