function [zirconStats, varargout] = zirconXtal_testIndConstGrowth(numSims, nucleationType, growthType, armorProb, zirconShape, uncertainties, timeSteps, useZirconComp, tests2perform, newAxes)
% function to generate synthetic zircon populations that crystallize over
% arbitrary duration 0 - 1
% input variables:
%
% numSims - number of synthetic zircons to generate
%
% nucleationType - zircon nucleation distribution. Options currently are: 
%       0 - instantaneous
%       1 - uniform
%       2 - normal (centered at 0.5, 1 sigma = 0.5, truncated at [0,1]
%       3 - beta distribution (early peak, asymptotes to 0)
%       4 - beta distribution (late peak)
%       5 - triangular, centered at 0.5
%       6 - linearly increasing
%       7 - linearly decreasing
%       8 - exponential
%
% growthType - zircon growth curves. Options currently limited to:
%       0 - instanaeous (cludged by setting armoring to always occur)
%       1 - uniform
%       2 - linear increasing
%       3 - linear decreasing
%       4 - exponential
%       (note, model assumes that growth defines total zircon volume, is
%       distributed over all grains that are nucleated and are not armored)
%       
% armorProb - probability that armoring of a given grain occurs each time
% step. Is scaled so that probability is constant indepedent of time step. 
%
% zirconShape - variable to define zircon shape. If zirconShape==1, zircon
% are modeled as spheres. For all other values, zircon is modeled as
% cylindrical shells with height = radius*zirconShape
%
% uncertainties, scaled as a portion of total duration (default = 0)
%
% timeSteps - number of time steps between 0 & 1 (default = 1e4)
%
% tests2perform - optional input to make additional tests after
% zircons are generated. If no values given, user will be prompted to
% select these options.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This version defines a single growth curve for every grain, which is
% constant at any given time (so just one single curve defines all growth)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


global MAKE_NEW_AXES
% this global variable keeps track of whether we should make a new figure
% or use whatever is already active. kind of lazy but works
if nargin<10
    MAKE_NEW_AXES = true;
else
    MAKE_NEW_AXES = newAxes;
end

if nargin<6
    uncertainties=0;
end

if nargin<7
    timeSteps = 1e4;
end

if nargin<8
    useZirconComp = false;
end


if nargin<9
    plotInds = {'Sample Durations from avg. ages' % 0
        'Sample Durations from avg. ages + uncertainties' % 1
        'Zircon Age Map' % 2
        'Zircon spot age curves' % 3
        'Zircon fragment age curves' % 4
        'Paper Figure 1' % 5
        'Paper Figure XX v. of sample durations' % 6
        'Paper Figure 2 zircon growth duration duration vs uncertainty' % 7
        'Total age histogram/pdf' % 8
        'CSD' % 9
        };
       
    [tests2perform,confirmed] = listdlg('ListString',plotInds, 'PromptString', 'Select plots to make', 'ListSize', [300, 150]);
    tests2perform = tests2perform-1;
    
    if confirmed==0
        disp('No additional processing requested, will return basic stats')
    end
end

if nucleationType==0 % need a workaround for instantaneous nucleation
    nucleationDist = zeros(numSims,1);
else
    nucleationRate = setNucleationDist(nucleationType);
    nucleationDist = random(nucleationRate, [numSims, 1]);
end

if growthType==0 % to get "instantaneous growth" we set growth rate to uniform, and armoring prob to 100% (so everything is immediately armored after nucleating)
    growthRate = setGrowthDist(1, timeSteps);
    armorProb = 1.01*timeSteps;
else
    growthRate = setGrowthDist(growthType, timeSteps);
end

if length(armorProb)==1
    armoringRate = ones(timeSteps+1,1)*armorProb/timeSteps; % keep this ind. of time steps
elseif length(armorProb)==2
    armoringRate = linspace(armorProb(1), armorProb(2), timeSteps+1)/timeSteps;
elseif length(armorProb)==timeSteps+1
    armoringRate = armorProb/timeSteps;
else
    error('Unexpected input for armoring rate')
end

    
zirconNucleation = false(size(nucleationDist));
armored = false(size(nucleationDist));

zirconGrowth = zeros(numSims, timeSteps);

for n = 1:timeSteps+1
    
    tStep = (n-1)/timeSteps;
    
    zirconNucleation(nucleationDist<=tStep & ~armored) = true;

    zirconGrowth(zirconNucleation,n) = growthRate(n);
    zirconGrowth(~zirconNucleation,n) = 0;
        
    armorEvents = rand(numSims,1)<armoringRate(n);
    armored(~armored & zirconNucleation & armorEvents) = true;
    
    zirconNucleation(armored) = false;
end

[~,zirconGrowthStart] = max(zirconGrowth>0,[],2);
zirconGrowthStart = (zirconGrowthStart-1)/timeSteps;

[~,zirconGrowthEndFlip]   = max(fliplr(zirconGrowth)>0, [], 2);
zirconGrowthEnd = (timeSteps+1-zirconGrowthEndFlip)/timeSteps;

zirconGrowthDur   = zirconGrowthEnd-zirconGrowthStart;

[zirconR1Inst, zirconR2Inst] = calculateZirconRad(zirconGrowth, zirconShape);
%zirconRad = (cumsum(zirconGrowth,2)*3/4/pi).^(1/3);
%zirconRadInst = diff(zirconRad, 1, 2);

if useZirconComp
    if length(useZirconComp) == 2
        meltUrange = useZirconComp;
    else
        meltUrange = [0.5 50];
    end
    zirconMeltD = 200;
    minAbsAge = 100e6;
    duration = 1e6;
    
    lambda = 1.55125e-10; % 238-206 decay constant, inv. years
    %lambda = 9.8485e-10; % 235-207 decay constant, inv. years
    
    zirconUscale = uraniumContents(meltUrange, zirconMeltD, timeSteps);
    zirconFinalComp = calculateDecay(zirconUscale, minAbsAge, duration, lambda);
    zirconMean_U = sum(zirconGrowth.*zirconFinalComp.U238,2)./sum(zirconGrowth,2);
    zirconMean_Pb= sum(zirconGrowth.*zirconFinalComp.Pb206,2)./sum(zirconGrowth,2);
    
    zirconMeanAge = log(zirconMean_Pb./zirconMean_U + 1)/lambda;
    zirconFinalComp.lambda = lambda;
else
    zirconMeanAge = sum(zirconGrowth.*(0:1/timeSteps:1),2)./sum(zirconGrowth,2);
    zirconFinalComp.lambda = nan;
end

zirconMeanAge_wUncertainties = zirconMeanAge + randn(size(zirconMeanAge))*uncertainties;

zirconStats.start = zirconGrowthStart;
zirconStats.end = zirconGrowthEnd;
zirconStats.dur = zirconGrowthDur;

zirconStats.meanAge = zirconMeanAge;
zirconStats.meanAge_plusError = zirconMeanAge_wUncertainties;

zirconStats.zirconR1Inst = zirconR1Inst;
zirconStats.zirconR2Inst = zirconR2Inst;

%% START ADDITIONAL POST-PROCESSING TESTS

if ismember(0, tests2perform)
    createAvgAgeDurations(zirconMeanAge);
end

if ismember(1, tests2perform)
    createAvgAgeDurationsUncertainties(zirconMeanAge);
end

if ismember(2, tests2perform)
    if size(zirconR1Inst,1)>1
        disp('Lots of zircons, just making plots for very first one!')
    end
    makeZirconAgeMap(zirconShape, zirconR1Inst(1,:), zirconR2Inst(1,:), zirconFinalComp);
end

if ismember(3, tests2perform)
    if size(zirconR1Inst,1)>1
        disp('Lots of zircons, just making plots for very first one!')
    end
    makeZirconSpotAgePlot(zirconR1Inst(1,:), zirconR2Inst(1,:), timeSteps, zirconMeanAge(1), zirconFinalComp);
end

if ismember(4, tests2perform)

    if size(zirconR1Inst,1)>1
        disp('Lots of zircons, just making plots for very first one!')
    end
    
    calculateZirconSplits(zirconR1Inst(1,:), zirconR2Inst(1,:), timeSteps, zirconMeanAge(1), zirconFinalComp);    
    
end

if ismember(5, tests2perform)
    makeZirconAgeMap(zirconShape, zirconR1Inst(1,:), zirconR2Inst(1,:), zirconFinalComp, 1);
 
    shortRadMax = max(cumsum(zirconR1Inst(1,:)))/max(cumsum(zirconR2Inst(1,:)));
    
    maxDist = 1-shortRadMax/2;
    
    viscircles([0, 0; 0, maxDist/4; 0 maxDist/2; 0 maxDist*3/4; 0 maxDist], shortRadMax/2*ones(5,1));
    
    makeZirconSpotAgePlot(zirconR1Inst(1,:), zirconR2Inst(1,:), timeSteps, zirconMeanAge(1), zirconFinalComp);
    
    calculateZirconSplits(zirconR1Inst(1,:), zirconR2Inst(1,:), timeSteps, zirconMeanAge(1), zirconFinalComp);
    axis([0 1 0 1]);
end

if ismember(6, tests2perform)
    createAvgAgeDurations(zirconMeanAge);
    legend off
    a1 = gca;
    a1.Color = 'none';
    a1Pos = a1.Position;
    a2 = axes('OuterPosition', [a1Pos(1)-0.02 a1Pos(2)+a1Pos(4)-0.13 0.15 0.15]);
    histogram(a2, zirconMeanAge, 0:0.05:1, 'FaceColor', 'none');
    a2.PlotBoxAspectRatio = [1 1 1];
    a2.Color = 'none';
    axis(a2, 'tight')
    a2.YAxis.Visible = 'off';
    box(a2, 'off')
    
    switch nucleationType
        case 1
            insetTitle = 'uniform';
        case 2
            insetTitle = 'trunc. norm.';
        case 4
            insetTitle = 'beta';
            a2.Position(2) = a1Pos(2)+a1Pos(4)-0.145;
        case 6
            insetTitle = 'lin. dec.';
            a2.Position(2) = a1Pos(2)+a1Pos(4)-0.145;
    end    
    title(insetTitle)
    
    axes(a1);
end

if ismember(7, tests2perform)

    instAges = linspace(1,0,1+timeSteps);
    
    relativeRadii = [0.1 0.25, 0.5 1];
    
    relativeSpotDepth=1;
    
    shortRadTotal = cumsum(zirconR1Inst(1,:));
    longRadTotal  = cumsum(zirconR2Inst(1,:));
    
    spotRadius = relativeRadii*shortRadTotal(end);
    spotDepth = spotRadius*relativeSpotDepth; % assume that spot depth is equal to radius?
    
    outerMostSpotCenter = longRadTotal(end) - spotRadius;
    spotCenters = [zeros(size(outerMostSpotCenter)); outerMostSpotCenter];
    
    for i = 1:length(spotRadius)
        
        tempRad = spotRadius(i);
        tempDepth = spotDepth(i);
        tempSpotCenters = spotCenters(:,i);
        
        for j = 1:length(tempSpotCenters)
            tempCenter = tempSpotCenters(j);
            for n = length(shortRadTotal):-1:1
                spotVolumeTotal(n) = rectangleCylinderIntersectVolumes(tempRad, tempDepth, 2*longRadTotal(n), shortRadTotal(n), tempCenter, 0);
            end
            spotVolumeNet = [spotVolumeTotal(1), diff(spotVolumeTotal)];
            if isnan(zirconFinalComp.lambda)
                spotAge(j,i) = sum(instAges.*spotVolumeNet)/sum(spotVolumeNet); %#ok<*AGROW>
            else
                spotU = sum(finalComp.U238.*spotVolumeNet)/sum(spotVolumeNet);
                spotPb= sum(finalComp.Pb206.*spotVolumeNet)/sum(spotVolumeNet);
                spotAge(j,i) = log(spotPb/spotU+1)/finalComp.lambda;
            end

        end
        
    end
    
    varargout = {relativeRadii, spotAge};
    
end

if ismember(8, tests2perform)
    avgAgeDistrubtionPDF(zirconMeanAge);
end

if ismember(9, tests2perform)
    CSD(sum(zirconR2Inst,2));
end

end % function end

%% FUNCTIONS TO DEFINE DISTRIBUTIONS

function nucleationFx = setNucleationDist(nType)

switch nType
    case 1 % uniform
        nucleationFx = makedist('Uniform', 0, 1);
    case 2 % normal (centered on 0.5, 0.5 sigma truncate at 0, 1)
        nucleationFx = makedist('Normal', 0.5, 0.25);
        nucleationFx = truncate(nucleationFx, 0, 1);
    case 3 % Beta distribution (arbitrary, but looks ~like the Brenhin model)
         nucleationFx = makedist('Beta', 2, 5);
    case 4 % mirrored Beta dist
        nucleationFx = makedist('Beta', 5, 2);
    case 5 % triangle, peak at 0.5
        nucleationFx = makedist('Triangular', 0, 0.5, 1);
    case 6 % linear increasing
        nucleationFx = makedist('Triangular', 0, 1, 1);
    case 7 % linear decreasing
        nucleationFx = makedist('Triangular', 0, 0, 1);
    case 8 % exponential
        nucleationFx = makedist('Exponential', 1/3);
        nucleationFx = truncate(nucleationFx, 0, 1);
    otherwise
        error('Undefined Nucleation Type')
end

end

function growthRateCurve = setGrowthDist(growthType, tSteps)

tVals = 0:1/tSteps:1;

switch growthType
    case 1 % uniform
        growthDist = makedist('Uniform', 0, 1);
    case 2 % linear increasing
        growthDist = makedist('Triangular', 0, 1, 1);
    case 3 % linear decreasing
        growthDist = makedist('Triangular', 0, 0, 1);
    case 4 % exponential
        growthDist = makedist('Exponential', 1/3);
        growthDist = truncate(growthDist, 0, 1);
    case 5 % beta dist
        growthDist = makedist('Beta', 2, 5);
    otherwise
        error('Undefined Growth Curve')
end

growthRateCurve = pdf(growthDist, tVals);

%growthRateCurve = 1-tVals.^(1/4);

%growthRateCurve = growthRateCurve/sum(growthRateCurve); % normalize

end

function [zirconR1Inst, zirconR2Inst] = calculateZirconRad(growthVolumes, shape)

if shape==1 % treat zircon as sphere
    zirconRad = (cumsum(growthVolumes,2)*3/4/pi).^(1/3);
elseif shape<0  % treat zircon as cylinder with height scaled as radius * shape
    zirconRad = (cumsum(growthVolumes,2)/-shape/pi).^(1/3);
    shape = -shape;
else % square prism, aspect ratio set by shape value
    zirconRad = (cumsum(growthVolumes,2)/shape).^(1/3);
end
    
zirconR1Inst = [zirconRad(:,1), diff(zirconRad,1,2)]/2;

zirconR2Inst = zirconR1Inst*shape;

end

function zirconU = uraniumContents(Ubounds, zirconD, tSteps)

% for now, this is just a simple model that has the zircon U contents vary
% linearly from initial U contents at t = 0 to final U contents at t = 1. U
% contents are set by first and second values of Ubounds, and zircon comp
% is calculated assuming zircon/melt distribution coefficient zirconD

initUcontent = Ubounds(1);
finalUcontent = Ubounds(2);

%meltUcontent = zeros(1,1+tSteps);
%meltUcontent(1,1:floor(length(meltUcontent)/2)) = initUcontent;
%meltUcontent(1,ceil(length(meltUcontent)/2):end) = finalUcontent;

meltUcontent = linspace(initUcontent, finalUcontent, 1+tSteps);

zirconU = zirconD*meltUcontent;

end

function finalZirconComp = calculateDecay(U238_i, minAbsAge, duration, lambda)

    ageRange = linspace(minAbsAge+duration, minAbsAge, length(U238_i));
    
    Pb206_f = U238_i.*(1-exp(-lambda*ageRange));
    U238_f = U238_i - Pb206_f;
    
    finalZirconComp.Pb206 = Pb206_f;
    finalZirconComp.U238  = U238_f;

end

%% FUNCTIONS FOR POST-PROCESSING

function avgAgeDistrubtionPDF(zirconAges)

global MAKE_NEW_AXES

if MAKE_NEW_AXES
    hold on; box on; axis square
end

coI = get(gca, 'ColorOrderIndex');

h1 = histogram(zirconAges,0:0.01:1, 'Normalization', 'pdf');
hold on
%ksdensity(zirconAges, 0:0.01:1, 'Support', [-0.001 1.0001])

co = get(gca, 'ColorOrder');
%numPlots = get(gca, 'Children');

h1.DisplayStyle = 'stairs';
h1.EdgeColor = co(coI,:);
h1.LineWidth = 1;

%if mod(length(numPlots),2)==0
%    h1.LineStyle = '--';
%    set(gca, 'ColorOrderIndex', coI+1);
%else
%    set(gca, 'ColorOrderIndex', coI);
%end




set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

%ylabel('Probability Density')
%xlabel('Zircon Age')

end

function CSD(zirconLengths)

global MAKE_NEW_AXES

if MAKE_NEW_AXES
    hold on; box on; axis square
end


h1 = histogram(zirconLengths);
hold on
%ksdensity(zirconAges, 0:0.01:1, 'Support', [-0.001 1.0001])

h1.DisplayStyle = 'stairs';
h1.EdgeColor = 'k';
h1.LineWidth = 1;

set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12, 'Yscale', 'log')

%ylabel('Probability Density')
%xlabel('Zircon Age')

end

function createAvgAgeDurations(zirconAges)

global MAKE_NEW_AXES

sampleNums = [3 5 7 10 20];

if MAKE_NEW_AXES
    figure; hold on; box on; axis square
end

for n = 1:length(sampleNums)
   sampleSets = randi(length(zirconAges), [5e4, sampleNums(n)]);
   sampleAvgAges = zirconAges(sampleSets);
   durations = range(sampleAvgAges,2);
   histogram(durations,0:0.01:1, 'Normalization', 'pdf');
end

legendTxt = sprintf('n = %i\n', sampleNums);
legendTxt = splitlines(legendTxt(1:end-1));

l1 = legend(legendTxt, 'Location', 'northwest');
l1.Title.String = 'Sample Size';

set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

ylabel('Probability Density')
xlabel('Measured Duration')

end

function createAvgAgeDurationsUncertainties(zirconAges)

uncertainties = [0.01:0.01:0.3 0.4:0.1:1 1.5 2];

sampleSize = [3 5 7 10];

figure;

medianDuration = zeros(length(sampleSize), length(uncertainties));

tiledlayout(2,2,'Padding', 'compact', 'TileSpacing', 'compact')

for m = 1:length(sampleSize)
    nexttile; hold on; box on; axis square
    for n = 1:length(uncertainties)
        sampleSets = randi(length(zirconAges), [5e4, sampleSize(m)]);
        sampleAvgAges = zirconAges(sampleSets);
        sampleAvgsAgesWUncertainties = sampleAvgAges+randn(5e4, sampleSize(m))*uncertainties(n);
        durations = range(sampleAvgsAgesWUncertainties,2);
        
        if ismembertol(uncertainties(n), [0.01 0.1 0.5 1 2])
            histogram(durations,0:0.05:10);
        elseif uncertainties(n)==0.1
            keyboard
        end
        
        medianDuration(m,n) = median(durations);
        
    end
    title(sprintf('Measured duration from %i samples', sampleSize(m)))
    
    set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

end

legendTxt = sprintf('\\sigma/duration = %g\n', [0.01 0.1 0.5 1 2]);
legendTxt = splitlines(legendTxt(1:end-1));

l1 = legend(legendTxt);
l1.Title.String = '1 Sigma uncertainties/real duration';
l1.Layout.Tile = 'South';
l1.Orientation = 'horizontal';

figure; hold on; axis square; box on
plot(uncertainties, medianDuration, 'LineWidth', 1)
xlabel('Relative Uncertainties')
ylabel('Median Measured Duration')

legendTxt = sprintf('n = %i\n', sampleSize);
legendTxt = splitlines(legendTxt(1:end-1));

l2 = legend(legendTxt, 'Location', 'northwest');
l2.Title.String = 'Number of samples';

set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

end

function makeZirconAgeMap(shape, shortRad, longRad, finalComp, useRelativeAxes)

if nargin<5
    useRelativeAxes = false;
end

shortRadTotal = cumsum(shortRad);
longRadTotal = cumsum(longRad);

if useRelativeAxes
    shortRadTotal = shortRadTotal/max(longRadTotal);
    longRadTotal = longRadTotal/max(longRadTotal);
end

figure;

if shape==1
    t1 = tiledlayout(1, 1);
    t1Text = 'Spherical Zircon Cross Section';
elseif shape<0
    t1 = tiledlayout(1, 2, 'Padding', 'compact', 'TileSpacing', 'compact');
    t1Text = 'Radial Cross Section';
else
    t1 = tiledlayout(1, 2, 'Padding', 'compact', 'TileSpacing', 'compact');
    t1Text = 'Square Cross Section';
end

nexttile; hold on
 % first plot radial component using shortRad (also only plot for
 % spherical grains)
%     angle = linspace(0,2*pi,360);
%     [R,A] = ndgrid(shortRadTotal/2,angle);
%     [X,Y] = pol2cart(A,R);
%     
%     radialColorNDGrid = repmat(linspace(0,1,length(shortRad))',1,360);
%     
%     pcolor(X,Y,radialColorNDGrid);
%     
%     axis equal
%     shading flat
%     contour(X,Y,radialColorNDGrid, 0.2:0.2:1, 'LineColor', 'k');    
if ~isnan(finalComp.lambda)
    localAges = log(finalComp.Pb206./finalComp.U238 + 1)/finalComp.lambda;    
    colormap = parula;
    ageMap = interp1(linspace(min(localAges), max(localAges), length(colormap)), colormap, localAges); 
    %ageMap = interp1(linspace(min(finalComp.Pb206), max(finalComp.Pb206), length(colormap)), colormap, finalComp.Pb206);
else
    ageMap = flipud(parula(length(longRad)));
end

sphPosition = [-shortRadTotal; -shortRadTotal; shortRadTotal*2; shortRadTotal*2;];

if shape == 1 || shape<0
    for n = length(longRad):-1:1
        if mod((n-1)/(length(longRad)-1), 0.2) == 0
            rectangle('Position', sphPosition(:,n), 'Curvature', [1 1], 'FaceColor', ageMap(n,:), 'LineStyle', '-')
        else
            rectangle('Position', sphPosition(:,n), 'Curvature', [1 1],  'FaceColor', ageMap(n,:), 'LineStyle', 'none')
        end
    end
else
    for n = length(longRad):-1:1
        if mod((n-1)/(length(longRad)-1), 0.2) == 0
            rectangle('Position', sphPosition(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', '-')
        else
            rectangle('Position', sphPosition(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', 'none')
        end
    end
end
    
    axis equal
    
    title(t1Text);

    set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12); box on
    
if shape<0 % then do the cylindrical part which we can represent as a rectangle (but this is kind of horendous)
    nexttile; hold on
    
    rectPositions = [-shortRadTotal; -longRadTotal; shortRadTotal*2; longRadTotal*2];
    
    for n = length(longRad):-1:1
        if mod((n-1)/(length(longRad)-1), 0.2) == 0
            rectangle('Position', rectPositions(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', '-')
        else
            rectangle('Position', rectPositions(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', 'none')
        end
    end
    
    yline(0, 'Color', 'r', 'LineWidth', 2, 'LineStyle', '--');
    
    title('Altitudinal Cross Section')
    t1.Title.String = sprintf('Cylindrical zircon with height = radius x %i', shape);

    set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)
end

if shape>0 && shape~=1
    nexttile; hold on
    rectPositions = [-shortRadTotal; -longRadTotal; shortRadTotal*2; longRadTotal*2];
    
    for n = length(longRad):-1:1
        if mod((n-1)/(length(longRad)-1),0.2)==0
            rectangle('Position', rectPositions(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', '-')
        else
            rectangle('Position', rectPositions(:,n), 'FaceColor', ageMap(n,:), 'LineStyle', 'none')
        end
    end    
    
    yline(0, 'Color', 'r', 'LineWidth', 2, 'LineStyle', '--');
 
    title('Altitudinal Cross Section')
    t1.Title.String = sprintf('Square prism zircon with height = radius x %i', shape);
 
    set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

    
end

cb = colorbar;
cb.Layout.Tile = 'East';
if ~isnan(finalComp.lambda)
    caxis([min(localAges) max(localAges)])
else
    caxis([0 1])
end

axis equal; box on
    
linkaxes(t1.Children(isgraphics(t1.Children, 'Axes')))

set(gcf, 'Renderer', 'painters')

end

function makeZirconSpotAgePlot(r1, r2, tSteps, meanAge, finalComp, relativeSpotDepth)

if nargin<6
    relativeSpotDepth=1;
end

shortRadTotal = cumsum(r1);
longRadTotal  = cumsum(r2);

instAges = linspace(1,0,1+tSteps);

%instAges = 0:1/tSteps:1;
relativeRadii = [0.01, 0.1, 0.25, 0.5 1];
spotRadius = relativeRadii*shortRadTotal(end);

spotDepth = spotRadius*relativeSpotDepth; % assume that spot depth is equal to radius?

spotVolumeTotal = zeros(size(shortRadTotal));

figure; hold on

lineColors = get(gca, 'ColorOrder');

for i = 1:length(spotRadius)
    tempRad = spotRadius(i);
    tempDepth = spotDepth(i);
    spotCenters = linspace(0, (longRadTotal(end)-tempRad));
    spotAge = zeros(size(spotCenters));
    for j = 1:length(spotCenters)
        tempCenter = spotCenters(j);        
        for n = length(shortRadTotal):-1:1
            spotVolumeTotal(n) = rectangleCylinderIntersectVolumes(tempRad, tempDepth, 2*longRadTotal(n), shortRadTotal(n), tempCenter, 0);
        end       
        spotVolumeNet = [spotVolumeTotal(1), diff(spotVolumeTotal)];
        if isnan(finalComp.lambda)
            spotAge(j) = sum(instAges.*spotVolumeNet)/sum(spotVolumeNet);
        else
            spotU = sum(finalComp.U238.*spotVolumeNet)/sum(spotVolumeNet);
            spotPb= sum(finalComp.Pb206.*spotVolumeNet)/sum(spotVolumeNet);
            spotAge(j) = log(spotPb/spotU+1)/finalComp.lambda;
        end
        if sum(isnan(spotAge))>1
            keyboard
        end
    end
    
    pH(i) = plot(spotCenters/longRadTotal(end), spotAge, 'Color', lineColors(i,:), 'LineWidth', 1); %#ok<AGROW>
end

%% add in a very lazy second set of data for 'SIMS'
relativeSpotDepth = 1e-2;
spotDepth = spotRadius*relativeSpotDepth;

for i = 1:length(spotRadius)
    tempRad = spotRadius(i);
    tempDepth = spotDepth(i);
    spotCenters = linspace(0, (longRadTotal(end)-tempRad));
    spotAge = zeros(size(spotCenters));
    for j = 1:length(spotCenters)
        tempCenter = spotCenters(j);        
        for n = length(shortRadTotal):-1:1
            spotVolumeTotal(n) = rectangleCylinderIntersectVolumes(tempRad, tempDepth, 2*longRadTotal(n), shortRadTotal(n), tempCenter, 0);
        end       
        spotVolumeNet = [spotVolumeTotal(1), diff(spotVolumeTotal)];
        if isnan(finalComp.lambda)
            spotAge(j) = sum(instAges.*spotVolumeNet)/sum(spotVolumeNet);
        else
            spotU = sum(finalComp.U238.*spotVolumeNet)/sum(spotVolumeNet);
            spotPb= sum(finalComp.Pb206.*spotVolumeNet)/sum(spotVolumeNet);
            spotAge(j) = log(spotPb/spotU+1)/finalComp.lambda;
        end
        if sum(isnan(spotAge))>1
            keyboard
        end
    end
    
    plot(spotCenters/longRadTotal(end), spotAge, '--', 'Color', lineColors(i,:), 'LineWidth', 1);
end

yline(meanAge, 'k--', 'Mean Grain Age', 'LabelVerticalAlignment', 'bottom', 'LabelHorizontalAlignment', 'left', 'LineWidth', 1);
xlabel('Relative Distance of spot center from grain center')
ylabel('Age')

legendTxt = sprintf('%0.2fx\n', relativeRadii);
legendTxt = splitlines(legendTxt(1:end-1));

l1 = legend(pH, legendTxt);
l1.Title.String = {'Spot radius relative to', 'zircon short axis'};

if isnan(finalComp.lambda)
    l1.Location = 'southwest';
else
    l1.Location = 'northeast';
end

box on; axis square

set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

end

function calculateZirconSplits(r1, r2, tSteps, meanAge, finalComp)
    
shortRadTotal = cumsum(r1);
longRadTotal  = cumsum(r2);

instAges = linspace(1,0,1+tSteps);

smallVolumeTotal = zeros(size(shortRadTotal));
bigVolumeTotal = zeros(size(shortRadTotal));
intVolumeTotal = zeros(size(shortRadTotal));

figure; hold on

breakPoint = linspace(0, longRadTotal(end), 1e3);

smallAge = zeros(size(breakPoint));
bigAge = zeros(size(breakPoint));
intAge = zeros(size(breakPoint));

for j = 1:length(breakPoint)
    for n = length(shortRadTotal):-1:1
        smallVolumeTotal(n) = rectangleIntersectVolumes(inf, inf, breakPoint(j), inf, longRadTotal(n), shortRadTotal(n));
        bigVolumeTotal(n) = rectangleIntersectVolumes(inf, inf, -inf, breakPoint(j), longRadTotal(n), shortRadTotal(n));
        intVolumeTotal(n) = rectangleIntersectVolumes(inf, inf, -breakPoint(j), breakPoint(j), longRadTotal(n), shortRadTotal(n));        
    end
    
    smallVolumeNet = [smallVolumeTotal(1) diff(smallVolumeTotal)];
    bigVolumeNet = [bigVolumeTotal(1) diff(bigVolumeTotal)];
    intVolumeNet = [intVolumeTotal(1) diff(intVolumeTotal)];
    
    if isnan(finalComp.lambda)
        smallAge(j) = sum(instAges.*smallVolumeNet)/sum(smallVolumeNet);
        bigAge(j) = sum(instAges.*bigVolumeNet)/sum(bigVolumeNet);
        intAge(j) = sum(instAges.*intVolumeNet)/sum(intVolumeNet);
    else
        smallU = sum(finalComp.U238.*smallVolumeNet)/sum(smallVolumeNet);
        smallPb= sum(finalComp.Pb206.*smallVolumeNet)/sum(smallVolumeNet);
        smallAge(j) = log(smallPb/smallU+1)/finalComp.lambda;
        
        bigU = sum(finalComp.U238.*bigVolumeNet)/sum(bigVolumeNet);
        bigPb= sum(finalComp.Pb206.*bigVolumeNet)/sum(bigVolumeNet);
        bigAge(j) = log(bigPb/bigU+1)/finalComp.lambda;        
        
        intU = sum(finalComp.U238.*intVolumeNet)/sum(intVolumeNet);
        intPb= sum(finalComp.Pb206.*intVolumeNet)/sum(intVolumeNet);
        intAge(j) = log(intPb/intU+1)/finalComp.lambda;   
        
    end
end

plot(breakPoint/max(breakPoint), smallAge, 'LineWidth', 1)
plot(breakPoint/max(breakPoint), bigAge, 'LineWidth', 1)
plot(breakPoint/max(breakPoint), intAge, 'LineWidth', 1)


y1 = yline(meanAge, 'k--', 'Mean Grain Age');
y1.LabelVerticalAlignment = "bottom";
y1.LineWidth = 1;
xlabel('Relative distance of break point from grain center')
ylabel('Age')

box on; axis square

legend({'Small Fragment', 'Large Fragment', 'Interior only'}, 'Location', 'southwest')

set(gca, 'LineWidth', 1, 'XColor', 'k', 'YColor', 'k', 'FontSize', 12)

end